/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.jonas.webapp.jonasadmin.joramplatform;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Set;
import java.util.Vector;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.jonas.lib.util.JonasObjectName;
import org.ow2.jonas.webapp.jonasadmin.JonasAdminJmx;
import org.ow2.jonas.webapp.jonasadmin.JonasManagementRepr;
import org.ow2.jonas.webapp.jonasadmin.WhereAreYou;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;


/**
 * @author Adriana Danes
 */

public class ListLocalDestinationsAction extends EditJoramBaseAction {

    ObjectName ejbServiceObjectName = null;
    String[] asParam = null;
    String[] asSignature = null;

    // --------------------------------------------------------- Public Methods
    public ActionForward executeAction(ActionMapping pMapping, ActionForm pForm
        , HttpServletRequest pRequest, HttpServletResponse pResponse)
        throws IOException, ServletException {
         // Current JOnAS server
        String jonasServerName =  m_WhereAreYou.getCurrentJonasServerName();
        String domainName = m_WhereAreYou.getCurrentDomainName();

        // Force the node selected in tree
        m_WhereAreYou.selectNameNode(getTreeBranchName(DEPTH_DOMAIN) + WhereAreYou.NODE_SEPARATOR
                + "joramplatform" + WhereAreYou.NODE_SEPARATOR
                + "joramlocalserver", true);

        // Get reference on EJB container service in order to determine dependencies
        // between the deployed EJBs and JMS destinations
        initRefs(domainName, jonasServerName);

        // Get all destinations
        String asDestName = null;
        try {
            // Object name used for JmslocalServer MBean
            ObjectName oObjectName = JonasObjectName.joramLocalServer();
            ArrayList alQueues = new ArrayList();
            // get queues
            Vector v = (Vector) getListAttribute(oObjectName, "LocalQueuesNames");
            if (v != null) {
                for (int i = 0; i < v.size(); i++) {
                    asDestName = (String) v.get(i);
                    ObjectName queueOn = new ObjectName("joram:type=JMSqueue,name=" + asDestName);
                    if (JonasAdminJmx.hasMBeanName(queueOn, jonasServerName)) {
                        boolean deps = hasDeps(asDestName, jonasServerName);
                        alQueues.add(new DestinationItem(asDestName, "queue", deps));
                    }
                }
            }
            // get topicss
            v = (Vector) getListAttribute(oObjectName, "LocalTopicsNames");
            ArrayList alTopics = new ArrayList();
            if (v != null) {
                for (int i = 0; i < v.size(); i++) {
                    asDestName = (String) v.get(i);
                    ObjectName topicOn = new ObjectName("joram:type=JMStopic,name=" + asDestName);
                    if (JonasAdminJmx.hasMBeanName(topicOn, jonasServerName)) {
                        boolean deps = hasDeps(asDestName, jonasServerName);
                        alQueues.add(new DestinationItem(asDestName, "topic", deps));
                    }
                }
            }

            // merge alQueues and alTopics in alDestinations
            ArrayList alDestinations = new ArrayList(alQueues);
            alDestinations.addAll(alTopics);

            pRequest.setAttribute("destinationsList", alDestinations);
        } catch (Throwable t) {
            return (treatError(t, pMapping, pRequest));
        }

        // Forward to the jsp.
        return (pMapping.findForward("Joram Destinations"));
    }

}
