package org.ow2.jonas.ws.axis2;

import org.ow2.jonas.lib.service.AbsServiceImpl;
import org.ow2.jonas.service.ServiceException;
import org.ow2.jonas.ws.axis2.jaxws.Axis2WSEndpoint;
import org.ow2.jonas.ws.jaxws.IJAXWSService;
import org.ow2.jonas.ws.jaxws.IWebServiceEndpoint;
import org.ow2.jonas.ws.jaxws.PortMetaData;
import org.ow2.jonas.ws.jaxws.WSException;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.api.struct.IJwsWebService;
import org.ow2.util.ee.metadata.war.api.IWarClassMetadata;
import org.ow2.util.event.api.IEventService;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import javax.jws.HandlerChain;
import javax.naming.NamingException;
import javax.naming.Reference;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

public class Axis2Service extends AbsServiceImpl implements IJAXWSService {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(Axis2Service.class);

    /**
     * The event service used to register listeners.
     */
    private IEventService eventService;


    @Override
    protected void doStart() throws ServiceException {
        // TODO Auto-generated method stub

    }

    @Override
    protected void doStop() throws ServiceException {
        // TODO Auto-generated method stub

    }

    public Reference createNamingReference(final IJaxwsWebServiceRef serviceRef) throws NamingException {
        return null;

    }

    public IWebServiceEndpoint createPOJOWebServiceEndpoint(final IWarClassMetadata metadata, final ClassLoader loader) throws WSException {
        // TODO Auto-generated method stub
        String classname = metadata.getJClass().getName().replace('/', '.');

        Class<?> klass = null;
        try {
            klass = loader.loadClass(classname);
        } catch (ClassNotFoundException e) {
            throw new WSException("Impossible to load the WebService class", e);
        }

        IJwsWebService ws = metadata.getJwsWebService();

        String wsdlName = ws.getWsdlLocation();

        // Load the WSDL if possible
        URL url = null;
        if (wsdlName != null) {
            url = loader.getResource(wsdlName);
            if (url == null) {
                try {
                    url = new URL(wsdlName);
                } catch (MalformedURLException ex2) {
                    url = loader.getResource("/" + wsdlName);
                }
            }
        }

        // find the URL pattern to use

        String serviceName = ws.getServiceName();

        String pattern = null;
        if (!"".equals(serviceName)) {
            // default pattern : /{service-name}
            pattern = "/" + serviceName;
        } else {
            // else /{class.name}Service
            pattern = "/" + klass.getSimpleName() + "Service";
        }

        // Creates PortMetaData to hold deployment/runtime information
        // about the web service
        PortMetaData pmd = new PortMetaData();

        pmd.setUrlPattern(pattern);

        StringBuffer handlerXML = new StringBuffer();
        try {
            HandlerChain hc = klass.getAnnotation(HandlerChain.class);

            if (hc != null) {
                URL handlerURL = klass.getClassLoader().getResource(hc.file());
                BufferedReader br = new BufferedReader(new InputStreamReader(handlerURL.openStream()));

                String temp = null;
                temp = br.readLine();
                while (temp != null) {
                    handlerXML.append(temp);
                    temp = br.readLine();
                }
                pmd.setHandlerXML(handlerXML.toString());
            }

        } catch (IOException e) {
            throw new WSException("Unable to Get the handler description file!", e);
        } finally {
            if(pmd.getHandlerXML() != null) {
                logger.debug("Found handlers in this package!");
            }
        }


        Axis2WSEndpoint endpoint = new Axis2WSEndpoint(klass, IWebServiceEndpoint.EndpointType.POJO, pmd, null);

        try {
            // init axis2
            endpoint.init();
        } catch (Exception e) {
            throw new WSException("Exception occurs when creating POJO endpoint!",e);
        }

        logger.info("Endpoint {0} inited", klass.getName());

        return endpoint;
    }


    public void bindEventService(final IEventService eventService) {
        logger.info("EventService instance {0}", eventService);
        this.eventService = eventService;
    }

    public void unbindEventService(final IEventService eventService) {
        this.eventService = null;
    }
}
