/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.ws.axis2.jaxws;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.net.HttpURLConnection;
import java.net.URI;
import java.util.List;
import java.util.Properties;

import javax.ejb.EJB;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.ws.Binding;
import javax.xml.ws.WebServiceException;
import javax.xml.ws.handler.Handler;

import org.apache.axiom.om.util.UUIDGenerator;
import org.apache.axis2.Constants;
import org.apache.axis2.addressing.AddressingHelper;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.OperationContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.description.TransportOutDescription;
import org.apache.axis2.engine.AxisEngine;
import org.apache.axis2.jaxws.binding.BindingImpl;
import org.apache.axis2.jaxws.binding.BindingUtils;
import org.apache.axis2.jaxws.description.EndpointDescription;
import org.apache.axis2.jaxws.description.impl.DescriptionUtils;
import org.apache.axis2.jaxws.description.xml.handler.HandlerChainsType;
import org.apache.axis2.jaxws.registry.FactoryRegistry;
import org.apache.axis2.jaxws.server.JAXWSMessageReceiver;
import org.apache.axis2.jaxws.server.endpoint.lifecycle.factory.EndpointLifecycleManagerFactory;
import org.apache.axis2.transport.RequestResponseTransport;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.transport.http.HTTPTransportUtils;
import org.apache.axis2.transport.http.TransportHeaders;
import org.apache.axis2.util.MessageContextBuilder;
import org.apache.http.HttpStatus;
import org.ow2.easybeans.container.session.stateless.StatelessSessionFactory;
import org.ow2.easybeans.resolver.api.EZBJNDIData;
import org.ow2.easybeans.resolver.api.EZBRemoteJNDIResolver;
import org.ow2.jonas.ws.axis2.easybeans.Axis2EjbMessageReceiver;
import org.ow2.jonas.ws.axis2.easybeans.Axis2EndpointLifecycleManagerFactory;
import org.ow2.jonas.ws.axis2.http.Axis2RequestResponseTransport;
import org.ow2.jonas.ws.axis2.http.Axis2TransportInfo;
import org.ow2.jonas.ws.axis2.http.AxisServiceGenerator;
import org.ow2.jonas.ws.axis2.http.WSDLQueryHandler;
import org.ow2.jonas.ws.jaxws.IWSRequest;
import org.ow2.jonas.ws.jaxws.IWSResponse;
import org.ow2.jonas.ws.jaxws.IWebServiceEndpoint;
import org.ow2.jonas.ws.jaxws.PortMetaData;
import org.ow2.jonas.ws.jaxws.WSException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class Axis2WSEndpoint implements IWebServiceEndpoint {

    private static final Log LOG = LogFactory.getLog(Axis2WSEndpoint.class);

    protected ConfigurationContext configurationContext;

    protected AxisService service;

    protected Binding binding;

    protected PortMetaData portMetaData;

    protected Class endpointClass;

    protected WSDLQueryHandler wsdlQueryHandler;

    protected StatelessSessionFactory factory;

    //protected JAXWSAnnotationProcessor annotationProcessor;

    /**
     * Describe the WS type (EJB/POJO).
     */
    private IWebServiceEndpoint.EndpointType type;

    public static final String END_POINT_TYPE = "EndPointType";

    public Axis2WSEndpoint(final Class endpointClass, final IWebServiceEndpoint.EndpointType type, final PortMetaData portMetaData,
            final StatelessSessionFactory factory) {
        this.endpointClass = endpointClass;
        this.portMetaData = portMetaData;
        this.type = type;
        this.factory = factory;
    }

    public void init() throws Exception {

        configurationContext = ConfigurationContextFactory.createBasicConfigurationContext("META-INF/axis2.xml");
        AxisServiceGenerator serviceGen = createServiceGenerator();

        // TODO: Get webservice from wsdl
        service = serviceGen.getServiceFromClass(this.endpointClass);
        service.setScope(Constants.SCOPE_APPLICATION);
        service.setEndpointName(service.getName());
        configurationContext.getAxisConfiguration().addService(service);
        this.wsdlQueryHandler = new WSDLQueryHandler(this.service);
        if (this.type == IWebServiceEndpoint.EndpointType.EJB) {
            this.configurationContext.setContextRoot(this.portMetaData.getContextRoot());
            this.configurationContext.setServicePath(this.portMetaData.getUrlPattern());
        }

        this.configureHandlers();

        FactoryRegistry.setFactory(EndpointLifecycleManagerFactory.class, new Axis2EndpointLifecycleManagerFactory());
    }

    protected AxisServiceGenerator createServiceGenerator() {
        AxisServiceGenerator serviceGenerator = null;
        if (this.type == IWebServiceEndpoint.EndpointType.POJO) {
            serviceGenerator = new AxisServiceGenerator();
        } else if (this.type == IWebServiceEndpoint.EndpointType.EJB) {
            serviceGenerator = new AxisServiceGenerator();
            Axis2EjbMessageReceiver messageReceiver = new Axis2EjbMessageReceiver(this, this.endpointClass, this.factory);
            serviceGenerator.setMessageReceiver(messageReceiver);
        }
        return serviceGenerator;
    }

    public EndpointType getType() {
        return this.type;
    }

    public void invoke(final IWSRequest request, final IWSResponse response) throws WSException {


        HttpServletRequest httpRequest = request.getAttribute(HttpServletRequest.class);
        HttpServletResponse httpResponse = request.getAttribute(HttpServletResponse.class);


        if ("GET".equals(httpRequest.getMethod())) {

            LOG.info("received a GET request");

            LOG.info("Query String:" + httpRequest.getQueryString());

            String query = httpRequest.getQueryString();
            if (query != null) {
                if (query.indexOf("wsdl") >= 0) {

                    try {
                        processListWSDL(request, response);
                    } catch (Exception e) {
                        throw new WebServiceException("process List WSDL failed", e);
                    }
                }
                return;
            }
        }



        MessageContext msgContext = new MessageContext();
        msgContext.setIncomingTransportName(Constants.TRANSPORT_HTTP);
        msgContext.setProperty(MessageContext.REMOTE_ADDR, request.getRemoteAddr());
        msgContext.setProperty(END_POINT_TYPE, this.getType());


        ClassLoader old = Thread.currentThread().getContextClassLoader();

        try {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());

            this.injectHandlers();

            TransportOutDescription transportOut = this.configurationContext.getAxisConfiguration().getTransportOut(
                    Constants.TRANSPORT_HTTP);
            TransportInDescription transportIn = this.configurationContext.getAxisConfiguration().getTransportIn(
                    Constants.TRANSPORT_HTTP);

            msgContext.setConfigurationContext(this.configurationContext);
            msgContext.setTransportIn(transportIn);
            msgContext.setTransportOut(transportOut);
            msgContext.setServiceGroupContextId(UUIDGenerator.getUUID());
            msgContext.setServerSide(true);
            msgContext.setAxisService(this.service);

            doService(request, response, msgContext);
        } catch (Throwable e) {
            String msg = "Exception occurred while trying to invoke service method doService()";
            LOG.error(msg, e);
            try {
                AxisEngine engine = new AxisEngine(this.configurationContext);

                msgContext.setProperty(MessageContext.TRANSPORT_OUT, response.getOutputStream());
                msgContext.setProperty(Constants.OUT_TRANSPORT_INFO, new Axis2TransportInfo(response));

                MessageContext faultContext = MessageContextBuilder.createFaultMessageContext(msgContext, e);
                // If the fault is not going along the back channel we should be
                // 202ing
                if (AddressingHelper.isFaultRedirected(msgContext)) {
                    response.setStatus(HttpURLConnection.HTTP_ACCEPTED);
                } else {
                    response.setStatus(HttpURLConnection.HTTP_INTERNAL_ERROR);
                }
                engine.sendFault(faultContext);
            } catch (Exception ex) {
                if (AddressingHelper.isFaultRedirected(msgContext)) {
                    response.setStatus(HttpURLConnection.HTTP_ACCEPTED);
                } else {
                    response.setStatus(HttpURLConnection.HTTP_INTERNAL_ERROR);
                    response.setHeader(HTTPConstants.HEADER_CONTENT_TYPE, "text/plain");
                    PrintWriter pw = null;
                    try {
                        pw = new PrintWriter(response.getOutputStream());
                    } catch (IOException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                    ex.printStackTrace(pw);
                    pw.flush();
                    LOG.error(msg, ex);
                }
            } finally {
                Thread.currentThread().setContextClassLoader(old);
            }

        }
    }

    protected void doService(final IWSRequest request, final IWSResponse response, final MessageContext msgContext) throws Exception {

        processPOSTRequest(request, response, this.service, msgContext);
        // Finalize response
        OperationContext operationContext = msgContext.getOperationContext();
        Object contextWritten = null;
        Object isTwoChannel = null;
        if (operationContext != null) {
            contextWritten = operationContext.getProperty(Constants.RESPONSE_WRITTEN);
            isTwoChannel = operationContext.getProperty(Constants.DIFFERENT_EPR);
        }

        if ((contextWritten != null) && Constants.VALUE_TRUE.equals(contextWritten)) {
            if ((isTwoChannel != null) && Constants.VALUE_TRUE.equals(isTwoChannel)) {
                response.setStatus(HttpURLConnection.HTTP_ACCEPTED);
                return;
            }
            response.setStatus(HttpURLConnection.HTTP_OK);
        } else {
            response.setStatus(HttpURLConnection.HTTP_ACCEPTED);
        }
    }

    // TODO: Get wsdl
    // protected void processGETRequest(final IWSRequest request, final
    // IWSResponse response, AxisService service,
    // MessageContext msgContext) throws Exception {
    // if (request.getRequestURI().getQuery() != null
    // && (request.getRequestURI().getQuery().startsWith("wsdl") ||
    // request.getURI().getQuery().startsWith("xsd"))) {
    // // wsdl or xsd request
    //
    // if (portInfo.getWsdlFile() != null && !portInfo.getWsdlFile().equals(""))
    // {
    // URL wsdlURL = AxisServiceGenerator.getWsdlURL(portInfo.getWsdlFile(),
    // configurationBaseUrl, classLoader);
    // this.wsdlQueryHandler
    // .writeResponse(request.getRequestURI().toString(), wsdlURL.toString(),
    // response.getOutputStream());
    // } else {
    // service.printWSDL(response.getOutputStream());
    // }
    // } else if (AxisServiceGenerator.isSOAP11(service)) {
    // response.setContentType("text/html");
    // PrintWriter pw = new PrintWriter(response.getOutputStream());
    // pw.write("<html><title>Web Service</title><body>");
    // pw.write("Hi, this is '" + service.getName() + "' web service.");
    // pw.write("</body></html>");
    // pw.flush();
    // } else {
    // // REST request
    // setMsgContextProperties(request, response, service, msgContext);
    //
    // String contentType =
    // request.getHeader(HTTPConstants.HEADER_CONTENT_TYPE);
    //
    // msgContext.setTo(new EndpointReference(request.getURI().toString()));
    //
    // msgContext.setProperty(MessageContext.TRANSPORT_OUT,
    // response.getOutputStream());
    // msgContext.setProperty(Constants.OUT_TRANSPORT_INFO, new
    // Axis2TransportInfo(response));
    //
    // InvocationResponse processed = RESTUtil.processURLRequest(msgContext,
    // response.getOutputStream(), contentType);
    //
    // if (!processed.equals(InvocationResponse.CONTINUE)) {
    // response.setStatus(HttpURLConnection.HTTP_OK);
    // String s = HTTPTransportReceiver.getServicesHTML(configurationContext);
    // PrintWriter pw = new PrintWriter(response.getOutputStream());
    // pw.write(s);
    // pw.flush();
    // }
    // }
    // }

    protected void setMsgContextProperties(final IWSRequest request, final IWSResponse response, final AxisService service,
            final MessageContext msgContext) throws IOException {
        msgContext.setProperty(MessageContext.TRANSPORT_OUT, response.getOutputStream());
        msgContext.setProperty(Constants.OUT_TRANSPORT_INFO, new Axis2TransportInfo(response));
        msgContext.setProperty(RequestResponseTransport.TRANSPORT_CONTROL, new Axis2RequestResponseTransport(response));
        msgContext.setIncomingTransportName(Constants.TRANSPORT_HTTP);

        HttpServletRequest servletRequest = request.getAttribute(HttpServletRequest.class);
        msgContext.setProperty(HTTPConstants.MC_HTTP_SERVLETREQUEST, servletRequest);
        msgContext.setProperty(Constants.Configuration.TRANSPORT_IN_URL, servletRequest.getRequestURI().toString());


        HttpServletResponse servletResponse = request.getAttribute(HttpServletResponse.class);
        msgContext.setProperty(HTTPConstants.MC_HTTP_SERVLETRESPONSE, servletResponse);

        ServletContext servletContext = request.getAttribute(ServletContext.class);
        msgContext.setProperty(HTTPConstants.MC_HTTP_SERVLETCONTEXT, servletContext);

        if (servletRequest != null) {
            msgContext.setProperty(MessageContext.TRANSPORT_HEADERS, new TransportHeaders(servletRequest));
        }

        if (this.binding != null) {
            msgContext.setProperty(JAXWSMessageReceiver.PARAM_BINDING, this.binding);
        }
    }

    protected void processPOSTRequest(final IWSRequest request, final IWSResponse response, final AxisService service,
            final MessageContext msgContext) throws Exception {
        HttpServletRequest httpRequest = request.getAttribute(HttpServletRequest.class);
        HttpServletResponse httpResponse = request.getAttribute(HttpServletResponse.class);

        String contentType = request.getHeader(HTTPConstants.HEADER_CONTENT_TYPE);
        String soapAction = request.getHeader(HTTPConstants.HEADER_SOAP_ACTION);
        if (soapAction == null) {
            soapAction = "\"\"";
        }

        ConfigurationContext configurationContext = msgContext.getConfigurationContext();
        configurationContext.fillServiceContextAndServiceGroupContext(msgContext);

        setMsgContextProperties(request, response, service, msgContext);

        HTTPTransportUtils.processHTTPPostRequest(msgContext,
                                                  request.getInputStream(),
                                                  response.getOutputStream(),
                                                  contentType,
                                                  soapAction,
                                                  new URI(httpRequest.getRequestURI()).getPath());
    }

    public void start() {
        // TODO Auto-generated method stub

    }

    public void stop() {
        // TODO Auto-generated method stub

    }

    public PortMetaData getPortMetaData() {
        // TODO Auto-generated method stub
        // return this.portMetaData;
        return this.portMetaData;
    }

    /*
     * Gets the right handlers for the port/service/bindings and performs injection.
     */
    protected void configureHandlers() throws Exception {
        EndpointDescription desc = AxisServiceGenerator.getEndpointDescription(this.service);
        if (desc == null) {
            this.binding = new BindingImpl("");
        } else {
            String xml = this.portMetaData.getHandlerXML();
            HandlerChainsType handlerChains = null;
            if (xml != null) {
                ByteArrayInputStream in = new ByteArrayInputStream(xml.getBytes("UTF-8"));
                handlerChains = DescriptionUtils.loadHandlerChains(in);
                desc.setHandlerChain(handlerChains);
            }

            this.binding = BindingUtils.createBinding(desc);

            ClassLoader old = Thread.currentThread().getContextClassLoader();

            Thread.currentThread().setContextClassLoader(this.endpointClass.getClassLoader());
            try{
                DescriptionUtils.registerHandlerHeaders(desc.getAxisService(), this.binding.getHandlerChain());
            } finally {
                Thread.currentThread().setContextClassLoader(old);
            }

        }
    }


    protected void injectHandlers() {
        List<Handler> handlers = this.binding.getHandlerChain();
        try {
            for (Handler<?> handler : handlers) {
                // axis2 finished WebServiceContext injection
                // so we just need to do injection for @EJB
                 injectEJBResources(handler);
            }
        } catch (Exception e) {
            throw new WebServiceException("Handler annotation failed", e);
        }
    }

    protected void injectEJBResources(final Object instance) {

        Field[] fieldArray = instance.getClass().getDeclaredFields();

        for(Field field : fieldArray){
            if(field.isAnnotationPresent(EJB.class)) {
                //LOG.info("@EJB exists!");
                EJB ejbResource = field.getAnnotation(EJB.class);

                String dependentEJBName = ejbResource.name();

               try
                {
                    System.setProperty(Context.INITIAL_CONTEXT_FACTORY,"org.objectweb.carol.jndi.spi.MultiOrbInitialContextFactory");

                    Context ictx = new InitialContext();

                    //Remote JNDI Resolver
                    EZBRemoteJNDIResolver jndiResolver = (EZBRemoteJNDIResolver)ictx.lookup("EZB_Remote_JNDIResolver");

                    String ejbResourceName = ((EZBJNDIData)(jndiResolver.getNames(dependentEJBName)).toArray()[0]).getName();

                    ClassLoader old = Thread.currentThread().getContextClassLoader();

                    try {
                          Thread.currentThread().setContextClassLoader(this.endpointClass.getClassLoader());

                          Object ejbObject = ictx.lookup(ejbResourceName);

                          boolean accessibility = field.isAccessible();

                          try {
                                field.setAccessible(true);
                                field.set(instance, ejbObject);
                              } catch (Exception e) {
                                throw e;
                              } finally {
                                field.setAccessible(accessibility);
                              }
                         } finally {
                              Thread.currentThread().setContextClassLoader(old);
                         }
                }
                catch(Exception e) {
                    // TODO Auto-generated catch block
                    throw new RuntimeException("Unable to inject the field!", e);
                }

           }

        }
    }

    private void processListWSDL(final IWSRequest request, final IWSResponse response) throws IOException, SAXException, ParserConfigurationException, TransformerException {
        HttpServletRequest httpRequest = request.getAttribute(HttpServletRequest.class);
        HttpServletResponse httpResponse = request.getAttribute(HttpServletResponse.class);

        response.setStatus(HttpStatus.SC_OK);

        response.setContentType("text/xml");

        String url = httpRequest.getRequestURL().toString();

        InputStream inStream = this.service.getClassLoader().getResourceAsStream("META-INF/" + this.endpointClass.getSimpleName() + ".wsdl");

        OutputStream outStream = response.getOutputStream();


        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();

        Document doc = null;
        doc = db.parse(inStream);
        doc.normalize();

        Element root = doc.getDocumentElement();

        NodeList nodeList = root.getChildNodes();

        ReplaceURL(nodeList,url);

        TransformerFactory factory = TransformerFactory.newInstance();
        Transformer transformer = factory.newTransformer();
        Properties properties = transformer.getOutputProperties();
        properties.setProperty(OutputKeys.ENCODING,"GB2312");
        properties.setProperty(OutputKeys.METHOD, "xml");
        transformer.setOutputProperties(properties);

        DOMSource source = new DOMSource(doc);

        StreamResult result = new StreamResult(outStream);

        transformer.transform(source, result);
    }

    public void ReplaceURL(final NodeList nodeList,final String url) {

        for(int i=0;i<nodeList.getLength();i++) {
            Node node = nodeList.item(i);
            if(node.getNodeType() == Node.ELEMENT_NODE &&
                    ( "service".equals(node.getNodeName()) || "wsdl:service".equals(node.getNodeName()) ) ) {

                NodeList nl1 = node.getChildNodes();
                for(int j=0;j<nl1.getLength();j++) {

                    Node nd1 = nl1.item(j);
                    if(nd1.getNodeType() == Node.ELEMENT_NODE &&
                       ("port".equals(nd1.getNodeName()) || "wsdl:port".equals(nd1.getNodeName()) ) ) {

                        LOG.info(nd1.getNodeName());

                        NodeList nl2 = nd1.getChildNodes();
                        for(int k=0;k<nl2.getLength();k++) {
                            Node nd2 = nl2.item(k);
                            if(nd2.getNodeType() == Node.ELEMENT_NODE &&
                               ( "soap:address".equals(nd2.getNodeName())
                               || "soap12:address".equals(nd2.getNodeName())
                               || "http:address".equals(nd2.getNodeName())) ) {

                                LOG.info(nd2.getNodeName());
                                nd2.getAttributes().getNamedItem("location").setNodeValue(url);
                                LOG.info(nd2.getAttributes().getNamedItem("location"));

                            }
                        }
                    }
                }
            }
        }
    }

}
